/* eslint-disable react/jsx-handler-names */
/* eslint-disable react/destructuring-assignment */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import Tooltip from 'primereact/tooltip';

export class ToggleButton extends Component {
	constructor(props) {
		super(props);
		this.state = {};
		this.toggle = this.toggle.bind(this);
		this.onFocus = this.onFocus.bind(this);
		this.onBlur = this.onBlur.bind(this);
		this.onKeyDown = this.onKeyDown.bind(this);
	}

	componentDidMount() {
		if (this.props.tooltip) {
			this.renderTooltip();
		}
	}

	componentDidUpdate(prevProps) {
		if (prevProps.tooltip !== this.props.tooltip) {
			if (this.tooltip) this.tooltip.updateContent(this.props.tooltip);
			else this.renderTooltip();
		}
	}

	componentWillUnmount() {
		if (this.tooltip) {
			this.tooltip.destroy();
			this.tooltip = null;
		}
	}

	toggle(e) {
		if (this.props.onChange) {
			this.props.onChange({
				originalEvent: e,
				value: !this.props.checked,
				stopPropagation: () => {},
				preventDefault: () => {},
				target: {
					name: this.props.name,
					id: this.props.id,
					value: !this.props.checked,
				},
			});

			this.input.focus();
		}
	}

	onFocus() {
		this.setState({ focused: true });
	}

	onBlur() {
		this.setState({ focused: false });
	}

	onKeyDown(event) {
		if (event.key === 'Enter') {
			this.toggle(event);
			event.preventDefault();
		}
	}

	renderTooltip() {
		this.tooltip = new Tooltip({
			target: this.container,
			content: this.props.tooltip,
			options: this.props.tooltipOptions,
		});
	}

	render() {
		const className = classNames('p-button p-togglebutton p-component', this.props.className, {
			'p-button-text-icon-left': this.props.onIcon && this.props.offIcon,
			'p-button-text-only':
				!this.props.onIcon && !this.props.offIcon && (this.props.onLabel || this.props.offLabel),
			'p-highlight': this.props.checked,
			'p-disabled': this.props.disabled,
			'p-focus': this.state.focused,
		});
		let iconStyleClass = null;

		if (this.props.onIcon || this.props.offIcon) {
			iconStyleClass = classNames(
				'p-c',
				this.props.checked ? this.props.onIcon : this.props.offIcon,
				{
					'p-button-icon-only':
						this.props.onIcon &&
						this.props.offIcon &&
						(!this.props.onLabel || !this.props.offLabel),
					'p-button-icon-left': this.props.onIcon && this.props.offIcon,
				}
			);
		}

		return (
			<div
				ref={el => (this.container = el)}
				id={this.props.id}
				className={className}
				style={this.props.style}
				onClick={this.toggle}
			>
				<div className="p-hidden-accessible">
					<input
						ref={el => (this.input = el)}
						type="checkbox"
						onFocus={this.onFocus}
						onBlur={this.onBlur}
						onKeyDown={this.onKeyDown}
						tabIndex={this.props.tabIndex}
						role="button"
						aria-pressed={this.props.checked}
						aria-label={this.props.ariaLabel}
						aria-labelledby={this.props.ariaLabelledBy}
						aria-describedby={this.props.ariaDescribedBy}
						required={this.props.required}
						aria-required={this.props.required}
					/>
				</div>
				{this.props.onIcon && this.props.offIcon && <span className={iconStyleClass} />}
				<span className="p-button-text p-unselectable-text">
					{this.props.checked ? this.props.onLabel : this.props.offLabel}
				</span>
			</div>
		);
	}
}

ToggleButton.defaultProps = {
	id: null,
	onIcon: null,
	offIcon: null,
	onLabel: 'Yes',
	offLabel: 'No',
	style: null,
	className: null,
	checked: false,
	tabIndex: 0,
	tooltip: null,
	tooltipOptions: null,
	ariaLabelledBy: null,
	onChange: null,
};

ToggleButton.propTypes = {
	ariaDescribedBy: PropTypes.string,
	ariaLabel: PropTypes.string,
	ariaLabelledBy: PropTypes.string,
	checked: PropTypes.bool,
	className: PropTypes.string,
	disabled: PropTypes.bool,
	id: PropTypes.string,
	name: PropTypes.string,
	offIcon: PropTypes.string,
	offLabel: PropTypes.string,
	onChange: PropTypes.func,
	onIcon: PropTypes.string,
	onLabel: PropTypes.string,
	required: PropTypes.bool,
	style: PropTypes.object,
	tabIndex: PropTypes.number,
	tooltip: PropTypes.string,
	tooltipOptions: PropTypes.object,
};

export default ToggleButton;
